@@ -8,6 +8,7 @@
 #' TRUE will look for any object called aoi on .GlobalEnv
 #' @author Guillermo Federico Olmedo
 #' @author Fonseca-Luengo, David
+#' @family remote sensing support functions
 #' @references 
 #' R. G. Allen, M. Tasumi, and R. Trezza, "Satellite-based energy balance for mapping evapotranspiration with internalized calibration (METRIC) - Model" Journal of Irrigation and Drainage Engineering, vol. 133, p. 380, 2007 \cr
 #' @export
@@ -82,6 +83,7 @@ loadImage <-  function(path = getwd(), sat="auto", aoi){
 #' @param aoi   area of interest to crop images, if waterOptions("autoAoi") == 
 #' TRUE will look for any object called aoi on .GlobalEnv
 #' @author Guillermo Federico Olmedo
+#' @family remote sensing support functions
 #' @export
 loadImageSR <-  function(path = getwd(),  aoi){
   files <- list.files(path = path, pattern = "_sr_band+[2-7].tif$", full.names = T)
@@ -99,6 +101,55 @@ loadImageSR <-  function(path = getwd(),  aoi){
   return(image_SR)}  
 
 
+#' Calculates radiance
+#' @description
+#' This function calculates radiance
+#' @param image.DN      raw image in digital numbers
+#' @param sat           "L7" for Landsat 7, "L8" for Landsat 8 or "auto" to guess from filenames 
+#' @param MTL           Landsat Metadata File
+#' @author Guillermo Federico Olmedo
+#' @author María Victoria Munafó
+#' @family remote sensing support functions
+#' @references 
+#' R. G. Allen, M. Tasumi, and R. Trezza, "Satellite-based energy balance for mapping evapotranspiration with internalized calibration (METRIC) - Model" Journal of Irrigation and Drainage Engineering, vol. 133, p. 380, 2007 \cr
+#' LPSO. (2004). Landsat 7 science data users handbook, Landsat Project Science Office, NASA Goddard Space Flight Center, Greenbelt, Md., (http://landsathandbook.gsfc.nasa.gov/) (Feb. 5, 2007) \cr
+#' @export
+calcRadiance <- function(image.DN, sat = "auto", MTL){
+  path <- getwd()
+  if(sat=="auto"){sat = getSat(path)} #DRY!
+  if(sat=="L8"){bands <- c(2:7, 10, 11)}
+  if(sat=="L7"){bands <- c(1:5,7, 6)}
+  if(missing(MTL)){MTL <- list.files(path = getwd(), pattern = "MTL.txt", full.names = T)}
+ 
+  MTL <- readLines(MTL, warn=FALSE)
+  ADD <- vector()
+  MULT <- vector()
+ 
+   for( i in 1:length(bands)){
+     
+     ADDstring <- paste0("RADIANCE_ADD_BAND_", bands[i])
+     ADDstring <- grep(ADDstring,MTL,value=TRUE)
+     ADD[i] <- as.numeric(regmatches(ADDstring, 
+                      regexec(text=ADDstring ,
+                         pattern="([-]*)([0-9]{1,5})([.]+)([0-9]+)"))[[1]][1])
+    
+     MULTstring <- paste0("RADIANCE_MULT_BAND_",bands[i])
+     MULTstring <- grep(MULTstring,MTL,value=TRUE)
+     MULT[i] <- as.numeric(regmatches(MULTstring, 
+                    regexec(text=MULTstring ,
+                    pattern="([0-9]{1,5})([.]+)([0-9]+)(E-)([0-9]+)"))[[1]][1])
+  
+   }
+  image <- image.DN * MULT + ADD
+  bandnames <- c("B", "G", "R", "NIR", "SWIR1", "SWIR2", "Thermal1")
+  if(sat=="L8"){bandnames <- c(bandnames, "Thermal2")}
+  image <- saveLoadClean(imagestack = image, 
+                            stack.names = bandnames, 
+                            file = "image_Rad", 
+                            overwrite=TRUE)
+  return(image)
+}
+
 #' Calculates Top of atmosphere reflectance
 #' @description
 #' This function calculates the TOA (Top Of Atmosphere) reflectance considering only the image metadata.
@@ -109,6 +160,7 @@ loadImageSR <-  function(path = getwd(),  aoi){
 #' @param MTL           Landsat Metadata File
 #' @author Guillermo Federico Olmedo
 #' @author Fonseca-Luengo, David
+#' @family remote sensing support functions
 #' @references 
 #' R. G. Allen, M. Tasumi, and R. Trezza, "Satellite-based energy balance for mapping evapotranspiration with internalized calibration (METRIC) - Model" Journal of Irrigation and Drainage Engineering, vol. 133, p. 380, 2007 \cr
 #'
@@ -170,6 +222,7 @@ calcTOAr <- function(image.DN, sat="auto",
 #' @param surface.model   rasterStack with DEM, Slope and Aspect. See surface.model()
 #' @author Guillermo Federico Olmedo
 #' @author Fonseca-Luengo, David 
+#' @family remote sensing support functions
 #' @references 
 #' Tasumi M.; Allen R.G. and Trezza, R. At-surface albedo from Landsat and MODIS satellites for use in energy balance studies of evapotranspiration Journal of Hydrolog. Eng., 2008, 13, (51-63) \cr
 #'
